// @ts-check
/* eslint-disable no-console */
const {existsSync, readFileSync} = require('fs')
const {extname} = require('path')
const {getSpawnPromise} = require('@s-ui/helpers/cli')
const {promisify} = require('util')
const exec = promisify(require('child_process').exec)

const GIT_IGNORE_PATH = `${process.cwd()}/.gitignore`

/**
 * Get lines of a text file as an array of strings
 * @param {String} path
 * @returns {Array<String>}
 */
const getFileLinesAsArray = path => (existsSync(path) ? readFileSync(path, 'utf8').split('\n').filter(Boolean) : [])

/**
 * Get as array .gitignore files and filter lines that are comments
 * @returns {Array<String>}
 */
const getGitIgnoredFiles = () => getFileLinesAsArray(GIT_IGNORE_PATH).filter(line => !line.startsWith('#'))

/**
 * Get from git status name of staged files
 * @param {object} params
 * @param {string[]} params.extensions Extensions list. Example: ['js', 'sass', 'css']
 * @param {string=} params.range Range of commits to lint
 * @param {boolean} params.staged If true, it will get staged files
 * @returns {Promise<string[]>} Array of file paths
 */
const getGitDiffFiles = async ({extensions, range = null, staged = true}) => {
  const command = ['git diff --name-only --diff-filter=d', range && `${range}`, staged && '--cached']
    .filter(Boolean)
    .join(' ')

  return exec(command).then(({stdout: summary = ''}) => getFilesFromDiff({summary, extensions}))
}

/**
 * Filter files from git diff status stdout
 * @param {object} params
 * @param {string[]} params.extensions Extensions list. Example: ['js', 'sass', 'css']
 * @param {string} params.summary stdout from git diff
 * @returns {string[]} Array of file paths
 */
const getFilesFromDiff = ({extensions, summary}) =>
  summary.split('\n').filter(file => extensions.includes(extname(file).substring(1)))

/**
 * Get the commit range depending on the CI used (Travis or GitHub Actions)
 * @returns {string|null} - Example: commit1...commit2
 */
const getCommitRange = () => {
  const {CI, GITHUB_EVENT_PATH, TRAVIS_COMMIT_RANGE: travisRange} = process.env
  // get commit range only for CI
  if (!CI) return null

  // Travis has a built-in environment variable that
  // always returns the commit range that we need
  if (travisRange) return travisRange

  if (GITHUB_EVENT_PATH) {
    const file = readFileSync(GITHUB_EVENT_PATH, 'utf8')
    const {after, before, pull_request: pullRequest} = JSON.parse(file)
    // get the correct commit range depending if
    // we're on a PR or a push to master
    const base = pullRequest?.base?.sha ?? before
    const head = pullRequest?.head?.sha ?? after

    if (base && head) {
      const commitRange = `${base}...${head}`
      console.log(`[sui-lint] Using commit range: ${commitRange}`)
      return commitRange
    }

    console.log('[sui-lint] No commit range found using GitHub Event from Actions')
  }

  return null
}

/**
 * Get files to lint according to command options
 * @param {Object} params
 * @param {string[]} params.extensions Extensions list: ['js', 'sass', 'css']
 * @param {string} params.defaultPattern Pattern with the files in case no other options are set
 * @param {boolean} params.staged get only staged files
 * @returns {Promise<string[]>} Array of file patterns
 */
const getFilesToLint = async ({extensions, defaultPattern, staged = false}) => {
  const range = getCommitRange()
  const getFromDiff = range || staged

  return getFromDiff ? getGitDiffFiles({extensions, range, staged}) : [defaultPattern]
}

/**
 * If --staged option is on, it will staged made changes
 * @param {Object} params
 * @param {Array<String>} params.extensions Extensions list: ['js', 'sass', 'css']
 * @param {boolean} params.staged get only staged files
 * @param {boolean} params.addFixes add modified files to stage
 * @returns {Promise}
 */
const stageFilesIfRequired = async ({extensions, staged, addFixes}) => {
  if (staged && addFixes) {
    const files = await getGitDiffFiles({extensions, staged})
    return getSpawnPromise('git', ['add', ...files])
  }
}

/**
 * Check if there're files to lint and output a message
 * @param {Object}                params
 * @param {String[]}              params.files Files to lint
 * @param {"JavaScript" | "SCSS"} params.language Language to lint
 * @param {String}                params.defaultPattern Default pattern to lint
 * @returns {boolean} If there's files to lint
 */
const checkFilesToLint = ({files, language, defaultPattern}) => {
  if (!files.length) {
    console.log(`[sui-lint] No ${language} files to lint`)
    return false
  }

  if (files.length === 1 && files[0] === defaultPattern) {
    console.log(`[sui-lint] Linting all ${language} files...`)
    return true
  }

  console.log(`[sui-lint] Linting ${files.length} ${language} files...`)
  return true
}

module.exports = {
  checkFilesToLint,
  getFileLinesAsArray,
  getFilesToLint,
  getGitIgnoredFiles,
  stageFilesIfRequired
}
